@@ -7,7 +7,7 @@ from more_itertools import roundrobin
 from itertools import chain
 from autorepr import autorepr, autotext
 from multiprocessing import Manager
-from ast import parse, Call, Name, UnaryOp, Constant, List, Dict, Return, For, Assign, If, Load, Store, keyword, Compare, BinOp, BoolOp, Add, Sub, Div, Mult, FloorDiv, Mod, MatMult, BitAnd, BitOr, BitXor, Invert, Not, UAdd, USub, LShift, RShift, And, Or, Eq, NotEq, Lt, Gt, LtE, GtE, Eq, NotEq, In, NotIn, Is, IsNot, ImportFrom, Attribute, IfExp, Subscript, Index, Tuple, Starred
+from ast import parse, Call, Name, UnaryOp, Constant, List, Dict, Return, For, Assign, If, Load, Store, keyword, Compare, BinOp, BoolOp, Add, Sub, Div, Mult, FloorDiv, Mod, MatMult, BitAnd, BitOr, BitXor, Invert, Not, UAdd, USub, LShift, RShift, And, Or, Eq, NotEq, Lt, Gt, LtE, GtE, Eq, NotEq, In, NotIn, Is, IsNot, ImportFrom, Attribute, IfExp, Subscript, Index, Tuple, Starred, With
 import ast
 import logging
 from importlib import import_module
@@ -16,6 +16,7 @@ from tx.functional.either import Left, Right, Either
 from .dependentqueue import DependentQueue
 from .stack import Stack
 from tx.readable_log import getLogger, format_message
+import typing as t
 
 
 logger = getLogger(__name__, logging.INFO)
@@ -98,6 +99,9 @@ def extract_expressions_to_assignments_in_statement(stmt, counter):
         expr, assigns = extract_expressions_to_assignments_in_expression(stmt.test, counter)
         stmt_eta = If(test=expr, body=extract_expressions_to_assignments(stmt.body, counter), orelse=extract_expressions_to_assignments(stmt.orelse, counter))
         return assigns + [stmt_eta]
+    if isinstance(stmt, With):
+        stmt_eta = With(items=stmt.items, body=extract_expressions_to_assignments(stmt.body, counter), type_comment=stmt.type_comment)
+        return [stmt_eta]
     elif isinstance(stmt, Return):
         ret = stmt.value
         expr, assigns = extract_expressions_to_assignments_in_expression(stmt.value, counter)
@@ -173,7 +177,7 @@ def extract_expressions_to_assignments_in_expression(expr, counter, in_assignmen
     else:
         return expr, []
 
-    logger.debug(format_message("extract_expressions_to_assignments", ast.dump(expr), (), {"expression": ast.dump(expr_eta), "assignments": [ast.dump(assign) for assign in assigns]}))
+    logger.debug(format_message("extract_expressions_to_assignments", lambda: ast.dump(expr), {"expression": lambda: ast.dump(expr_eta), "assignments": lambda: [ast.dump(assign) for assign in assigns]}))
     if in_assignment:
 #       logger.debug("in assignment")
         return expr_eta, assigns
@@ -226,9 +230,10 @@ def python_to_spec_seq(body, imported_names = {}):
         apps = [stmt for stmt in stmts if isinstance(stmt, Assign) and is_dynamic(stmt.value)]
         fors = [stmt for stmt in stmts if isinstance(stmt, For)]
         ifs = [stmt for stmt in stmts if isinstance(stmt, If)]
+        withs = [stmt for stmt in stmts if isinstance(stmt, With)]
         returns = [stmt for stmt in stmts if isinstance(stmt, Return)]
     
-        return python_to_top_spec(apps + fors + ifs + returns, imported_names2)
+        return python_to_top_spec(apps + fors + ifs + withs + returns, imported_names2)
     
     return let_spec(body, func)
 
@@ -243,8 +248,6 @@ def python_to_top_spec(body, imported_names):
             "sub": specs
         }
 
-EnvStack2 = Stack(set())
-
 def python_to_spec_in_top(stmt, imported_names):
     if isinstance(stmt, For):
         coll_name = python_ast_to_term(stmt.iter)
@@ -262,6 +265,11 @@ def python_to_spec_in_top(stmt, imported_names):
             "then": python_to_spec_seq(stmt.body, imported_names),
             "else": python_to_spec_seq(stmt.orelse, imported_names),
         }]
+    if isinstance(stmt, With):
+        return [{
+            "type": "seq",
+            "sub": list(chain.from_iterable(python_to_spec_in_top(stmt, imported_names) for stmt in stmt.body))
+        }]
     elif isinstance(stmt, Return):
         ret_val = stmt.value
         return [{
